@@ -20,6 +20,11 @@
 #endif
 #include <sys/wait.h>
 
+/* Needed on AIX 4.1 to get fd_set and friends. */
+#if HAVE_SYS_SELECT_H
+#    include <sys/select.h>
+#endif
+
 #include "inn/innconf.h"
 #include "inn/libinn.h"
 #include "inn/messages.h"
@@ -119,6 +124,7 @@ bool laxmid = false;
 
 /* Other default values. */
 bool Tracing = false;
+bool hasSentCapabilities = false;
 
 #ifdef DO_PERL
 bool PerlLoaded = false;
@@ -137,8 +143,8 @@ static char CMDfetchhelp[] = "[message-ID|number]";
 */
 static CMDENT CMDtable[] = {
     {"ARTICLE",      CMDfetch,        true,  1, 2,      true,  CMDfetchhelp               },
- /* Parse AUTHINFO in a special way so as to keep white spaces
-  * in usernames and passwords. */
+    /* Parse AUTHINFO in a special way so as to keep white spaces
+     * in usernames and passwords. */
     {"AUTHINFO",     CMDauthinfo,     false, 3, CMDany, false,
      "USER name|PASS password"
 #ifdef HAVE_SASL
@@ -170,8 +176,8 @@ static CMDENT CMDtable[] = {
     {"OVER",         CMDover,         true,  1, 2,      true,  "[range]"                  },
     {"POST",         CMDpost,         true,  1, 1,      true,  NULL                       },
     {"QUIT",         CMDquit,         false, 1, 1,      true,  NULL                       },
- /* SLAVE (which was ill-defined in RFC 977) was removed from the NNTP
-  * protocol in RFC 3977. */
+    /* SLAVE (which was ill-defined in RFC 977) was removed from the NNTP
+     * protocol in RFC 3977. */
     {"SLAVE",        CMD_unimp,       false, 1, 1,      true,  NULL                       },
 #ifdef HAVE_OPENSSL
     {"STARTTLS",     CMDstarttls,     false, 1, 1,      true,  NULL                       },
@@ -209,7 +215,7 @@ ExitWithStats(int x, bool readconf)
 
     GRPreport();
     if (ARTcount)
-        syslog(L_NOTICE, "%s exit articles %ld groups %ld", Client.host,
+        syslog(L_NOTICE, "%s exit articles %lu groups %ld", Client.host,
                ARTcount, GRPcount);
     if (POSTreceived || POSTrejected)
         syslog(L_NOTICE, "%s posts received %ld rejected %ld", Client.host,
@@ -318,11 +324,11 @@ CMDhelp(int ac UNUSED, char *av[] UNUSED)
         else
             Printf("  %s %s\r\n", cp->Name, cp->Help);
     }
-    if (PERMaccessconf && (VirtualPathlen > 0)) {
-        if (PERMaccessconf->newsmaster) {
+    if (PERMaccessconf != NULL) {
+        if (PERMaccessconf->newsmaster != NULL) {
             if (strchr(PERMaccessconf->newsmaster, '@') == NULL) {
                 Printf("Report problems to <%s@%s>.\r\n",
-                       PERMaccessconf->newsmaster, PERMaccessconf->domain);
+                       PERMaccessconf->newsmaster, PERMaccessconf->fromhost);
             } else {
                 Printf("Report problems to <%s>.\r\n",
                        PERMaccessconf->newsmaster);
@@ -331,11 +337,11 @@ CMDhelp(int ac UNUSED, char *av[] UNUSED)
             /* Sigh, pickup from newsmaster anyway. */
             if ((p = strchr(newsmaster, '@')) == NULL)
                 Printf("Report problems to <%s@%s>.\r\n", newsmaster,
-                       PERMaccessconf->domain);
+                       PERMaccessconf->fromhost);
             else {
                 q = xstrndup(newsmaster, p - newsmaster);
                 Printf("Report problems to <%s@%s>.\r\n", q,
-                       PERMaccessconf->domain);
+                       PERMaccessconf->fromhost);
                 free(q);
             }
         }
@@ -368,6 +374,9 @@ CMDcapabilities(int ac, char *av[])
         return;
     }
 
+    /* Keep in mind the client has used this command. */
+    hasSentCapabilities = true;
+
     Reply("%d Capability list:\r\n", NNTP_INFO_CAPABILITIES);
 
     Printf("VERSION 2\r\n");
@@ -402,7 +411,7 @@ CMDcapabilities(int ac, char *av[])
 #ifdef HAVE_OPENSSL
             } else {
 #    ifdef HAVE_SASL
-                /* Remove unsecure PLAIN, LOGIN and EXTERNAL SASL mechanisms,
+                /* Remove insecure PLAIN, LOGIN and EXTERNAL SASL mechanisms,
                  * if compiled with TLS support and a TLS layer is not active.
                  */
                 if (mechlist != NULL) {
@@ -1013,9 +1022,6 @@ main(int argc, char *argv[])
     message_handlers_warn(1, message_log_syslog_warning);
     message_handlers_notice(1, message_log_syslog_notice);
 
-    /* Initialize the character class tables for message-IDs. */
-    InitializeMessageIDcclass();
-
     if (!innconf_read(NULL))
         exit(1);
 
@@ -1078,7 +1084,8 @@ main(int argc, char *argv[])
         case 'r': /* Reject connection message. */
             Reject = xstrdup(optarg);
             break;
-        case 's': /* Unused title string. */
+        case 's': /* Unused title string - just to allocate space in command
+                     line to be filled by setproctitle. */
             break;
         case 't': /* Tracing. */
             Tracing = true;
@@ -1099,7 +1106,7 @@ main(int argc, char *argv[])
     if (innconf->nicennrpd != 0) {
         errno = 0;
         if (nice(innconf->nicennrpd) < 0 && errno != 0)
-            syswarn("could not nice to %ld", innconf->nicennrpd);
+            syswarn("could not nice to %lu", innconf->nicennrpd);
     }
 
     HISTORY = concatpath(innconf->pathdb, INN_PATH_HISTORY);
@@ -1174,7 +1181,9 @@ main(int argc, char *argv[])
 
         /* Detach. */
         if (!ForeGroundMode) {
-            daemonize("/");
+            if (daemon(0, 0) < 0) {
+                sysdie("cannot fork: %s", strerror(errno));
+            }
         }
 
         if (ListenPort == NNTP_PORT)
@@ -1502,7 +1511,7 @@ main(int argc, char *argv[])
             r = line_read(&NNTPline, timeout, &p, &len, &lenstripped);
             switch (r) {
             default:
-                syslog(L_ERROR, "%s internal %d in main", Client.host, r);
+                syslog(L_ERROR, "%s internal %u in main", Client.host, r);
                 goto fallthroughRTtimeout;
             case RTtimeout:
             fallthroughRTtimeout:
@@ -1593,6 +1602,9 @@ main(int argc, char *argv[])
                 /* Acknowledge the magic word from the Colossal Cave Adventure
                  * computer game. */
                 Reply("%d Nothing happens\r\n", NNTP_ERR_COMMAND);
+            } else if (strcasecmp(av[0], "MAXARTNUM") == 0) {
+                Reply("%d Wish I could be taught to count beyond %lu\r\n",
+                      NNTP_ERR_COMMAND, NNTP_MAXARTNUM);
             } else {
                 Reply("%d What?\r\n", NNTP_ERR_COMMAND);
                 if ((int) strlen(buff) > 40)
